import { jsx as _jsx, Fragment as _Fragment } from "react/jsx-runtime";
import { EditorView, keymap } from "@codemirror/view";
import { ReplayDataRecorder } from "@liqvid/recording";
import { bind } from "@liqvid/utils/misc";
import { icon } from "./icon.mjs";
// the actual thingy that gets exported
export class CodeRecorder extends ReplayDataRecorder {
    constructor() {
        super();
        bind(this, ["extension"]);
    }
    /**
     * Get a CodeMirror extension for recording.
     * @param specialKeys Map of key sequences to commands, e.g. {"Mod-Enter": "run"}.
     * @returns CodeMirror extension.
     */
    extension(specialKeys = {}) {
        // legacy
        if (specialKeys instanceof Array) {
            specialKeys = Object.fromEntries(specialKeys.map(key => [key, key]));
        }
        // record document changes
        const updateListener = EditorView.updateListener.of(update => {
            if (!this.manager || this.manager.paused || !this.manager.active)
                return;
            // get selection change (if any)
            const transactions = update.transactions
                .map(t => {
                if (!t.selection)
                    return null;
                const range = t.selection.ranges[0];
                return [range.anchor, range.head];
            })
                .filter(Boolean)
                .slice(-1);
            // empty events can break replay
            if (update.changes.empty && transactions.length === 0)
                return;
            this.capture(this.manager.getTime(), [
                update.changes.toJSON(),
                ...transactions
            ]);
        });
        // record special key presses
        const keyListener = keymap.of(Object.keys(specialKeys).map(key => ({
            key,
            run: () => {
                if (this.manager && this.manager.active && !this.manager.paused) {
                    this.capture(this.manager.getTime(), specialKeys[key]);
                }
                return false;
            }
        })));
        return [updateListener, keyListener];
    }
}
function KeySaveComponent(props) {
    return (_jsx(_Fragment, { children: _jsx("textarea", { readOnly: true, value: JSON.stringify(props.data) }) }));
}
export const CodeRecording = {
    enabled: () => true,
    icon,
    key: "codemirror",
    name: "Code",
    recorder: new CodeRecorder(),
    saveComponent: KeySaveComponent,
    title: "Record code"
};
