/// <reference types="node" />
import { ChangeSet, Text } from "@codemirror/state";
import type { EditorView } from "@codemirror/view";
import type { ReplayData } from "@liqvid/utils/replay-data";
import type { EventEmitter } from "events";
declare type Action = string | [changes: ChangeSet, selection?: [number, number]];
/** Reserved command for specifying file. */
export declare const selectCmd = "file:";
/**
 * Replay typing in CodeMirror.
 */
export declare function cmReplay({ data, handle, playback, start, view }: Omit<Parameters<typeof cmReplayMultiple>[0], "handle" | "views"> & {
    /**
     * Function for handling special commands.
     * @param cmd Command to handle.
     * @param doc CodeMirror document.
     */
    handle?: (cmd: string, doc: Text) => void;
    /** CodeMirror instance to sync with. */
    view: EditorView;
}): () => void;
/** Replay typing to several CodeMirror instances in parallel. */
export declare function cmReplayMultiple({ data, handle, playback, start, views }: {
    /** Recording data to replay. */
    data: ReplayData<Action>;
    /**
     * Function for handling special commands.
     * @param cmd Command to handle.
     * @param docs CodeMirror documents.
     */
    handle?: (cmd: string, docs: Record<string, Text>) => void;
    /** Playback to sync with. */
    playback: EventEmitter;
    /**
     * Time playback should start.
     * @default 0
     */
    start?: number;
    /** CodeMirror instances to sync with. */
    views: Record<string, EditorView>;
}): () => void;
export { fakeSelection } from "./fake-selection";
